/*
Copyright (c) 2004 Joseph Gleason

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

Current versions of this and other code can be downloaded at:
http://gleason.cc/

*/

#include "rwmonitor.h"

//Most comments are in header file

void rw_monitor_init(rw_monitor_t* R)
{
    R->WaitingReaders=0;
    R->RunningReaders=0;
    R->WaitingWriters=0;
    R->RunningWriters=0;
    pthread_mutex_init(&R->Mutex,NULL);
    pthread_cond_init(&R->Cond,NULL);
}

void rw_monitor_destroy(rw_monitor_t *R)
{
    pthread_mutex_destroy(&R->Mutex);
    pthread_cond_destroy(&R->Cond);
}

void rw_monitor_read(rw_monitor_t* R)
{
    pthread_mutex_lock(&R->Mutex);

    R->WaitingReaders++;

    while((R->WaitingWriters>0) || (R->RunningWriters>0))
    {
        pthread_cond_wait(&R->Cond,&R->Mutex);
    }

    R->WaitingReaders--;
    R->RunningReaders++;
    pthread_mutex_unlock(&R->Mutex);

}

void rw_monitor_unread(rw_monitor_t* R)
{
    pthread_mutex_lock(&R->Mutex);

    R->RunningReaders--;

    pthread_cond_broadcast(&R->Cond);

    pthread_mutex_unlock(&R->Mutex);
}
void rw_monitor_write(rw_monitor_t* R)
{
    pthread_mutex_lock(&R->Mutex);

    R->WaitingWriters++;

    while((R->RunningReaders>0) || (R->RunningWriters>0))
    {
        pthread_cond_wait(&R->Cond,&R->Mutex);
    }

    R->WaitingWriters--;
    R->RunningWriters++;

    pthread_mutex_unlock(&R->Mutex);

}
void rw_monitor_unwrite(rw_monitor_t* R)
{
    pthread_mutex_lock(&R->Mutex);

    R->RunningWriters--;

    pthread_cond_broadcast(&R->Cond);

    pthread_mutex_unlock(&R->Mutex);
}
