/*
Copyright (c) 2004 Joseph Gleason

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

Current versions of this and other code can be downloaded at:
http://gleason.cc/

*/

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "str2tok.h"

//Most documentation is in header file

int isoneof(char c, char* Delims, int DelimLen)
{
   int i;
   for(i=0; i<DelimLen; i++)
   {
      if (Delims[i]==c) return c;
   }
   return 0;
}

char** str2tok(char* s, char* Delims, int *StringCount, int MaxStrings, size_t MaxLength)
{
   char** StringArray;
   StringArray=(char**)malloc(sizeof(char*)*MaxStrings);
   size_t i;

   for(i=0; i<MaxStrings; i++)
   {
      StringArray[i]=NULL;
   }
   int SC=0; //String Count


   size_t Len;
   int DelimLen;

   Len=strlen(s);
   DelimLen=strlen(Delims);
   size_t CurrStrSize=0;
   for(i=0; i<Len; i++)
   {
      if (isoneof(s[i],Delims,DelimLen))
      {
         if (CurrStrSize>0)
         {
            StringArray[SC][CurrStrSize]=0;
            SC++;
            if (SC==MaxStrings){ i=Len; } //We are done
            CurrStrSize=0;
         }
      }
      else
      {
         if (CurrStrSize==0)
         {
            StringArray[SC]=(char*)malloc(MaxLength);
         }
         //This way, for an overly large string,
         //CurrStrSize will be MaxLength-1
         //Allowing for the above 'StringArray[SC][CurrStrSize]=0;'
         //to do the right thing
         if (CurrStrSize<MaxLength-1)
         {
            StringArray[SC][CurrStrSize]=s[i];
            CurrStrSize++;
         }
      }
   }

   if (CurrStrSize>0)
   {
      StringArray[SC][CurrStrSize]=0;
      SC++;
      if (SC==MaxStrings){ i=Len; } //We are done
      CurrStrSize=0;
   }

   *StringCount=SC;
   return StringArray;
}

void str2tok_cleanup(char** StringArray, int StringCount)
{
   int i;
   for(i=0; i<StringCount; i++)
   {
      free(StringArray[i]);
   }
   free(StringArray);
}

void strtrimws(char* s)
{
   if (strlen(s)==0) return;

   while(isspace(s[0]))
   {
      int i;
      for(i=0; i<strlen(s);i++)
      {
         s[i]=s[i+1];
      }
   }

   while(isspace(s[strlen(s)-1])) s[strlen(s)-1]=0;


}
